"""The backends module contains the backend classes for the xrdsum program."""
from __future__ import annotations

from ._base import XrdsumBackend
from ._cephfs import CephFSBackend
from ._hdfs import HDFSBackend
from ._http import HttpBackend
from ._posix import POSIXBackend

FILE_SYSTEMS: dict[str, type[XrdsumBackend]] = {
    "HDFS": HDFSBackend,
    "CephFS": CephFSBackend,
    "HTTP": HttpBackend,
    "POSIX": POSIXBackend,
}

__all__ = ["FILE_SYSTEMS"]



"""Implementation of the Compact Muon Solenoid (CMS) file catalog lookup (storage.xml) """
from __future__ import annotations

import re
from dataclasses import dataclass, field
from xml.dom.minidom import parse as parse_xml

from ._base import StorageCatalog


@dataclass
class CMSStorageCatalog(StorageCatalog):
    """CMS file catalog lookup"""

    config: str = "storage.xml"
    protocol: str = "direct"
    transformation_rules: list[tuple[re.Pattern[str], str]] = field(
        default_factory=list
    )

    def __init__(self, config: str | None = None, protocol: str | None = None) -> None:
        if config is not None:
            self.config = config
        if protocol is not None:
            self.protocol = protocol
        self.transformation_rules = self.__read_config()

    def __read_config(self) -> list[tuple[re.Pattern[str], str]]:
        """Reads the CMS storage catalog"""

        cms_rules: list[tuple[re.Pattern[str], str]] = []
        with parse_xml(self.config) as dom:
            rules = dom.getElementsByTagName("lfn-to-pfn")
            for rule in rules:
                if rule.getAttribute("protocol") != self.protocol:
                    continue
                pattern = re.compile(rule.getAttribute("path-match"))
                result = rule.getAttribute("result")
                cms_rules.append((pattern, result))
        return cms_rules

    def is_cms_path(self, path: str) -> bool:
        """Checks if the path is a CMS path"""
        cms_regex = re.compile(r"^/*(store.*)")
        match = cms_regex.match(path)
        if match:
            return True
        return False

    def lfn2pfn(self, lfn: str) -> str:
        """Converts logical filename to physical filename"""
        for rule in self.transformation_rules:
            pattern, result = rule
            match = pattern.match(lfn)
            if match is None:
                continue
            result = result.replace("$", "\\")
            return match.expand(result)
        return lfn



"""Definition of the Checksum protocol."""
from __future__ import annotations

from collections.abc import Iterable
from typing import Any, Protocol


class Checksum(Protocol):
    """Base protocol for checksum implementations."""

    name: str = "Unknown"
    value: str = "N/A"
    bytes_read: int = 0
    number_of_buffers_read: int = 0

    def int_to_hex(self, value: int) -> str:
        """Converts integer to hex representation"""
        raise NotImplementedError()

    def hex_to_int(self, value: str) -> int:
        """Converts hex representation to integer"""
        raise NotImplementedError()

    def calculate(self, file_buffer: Iterable[Any]) -> str:
        """Calculates the checksum"""
        raise NotImplementedError()



"""xrdsum.checksums package"""
from __future__ import annotations

from ._base import Checksum
from .adler32 import Adler32

AVAILABLE_CHECKSUM_TYPES = {
    "adler32": Adler32,
}
__all__ = ["Adler32", "Checksum", "AVAILABLE_CHECKSUM_TYPES"]



"""Module for calculating Adler32 checksums."""
from __future__ import annotations

import struct
import zlib
from collections.abc import Iterable
from typing import Any

from ..logger import APP_LOGGER_NAME, get_logger
from ._base import Checksum

log = get_logger(APP_LOGGER_NAME)


class Adler32(Checksum):
    """Adler32 checksum
    from https://github.com/snafus/cephsum/blob/master/cephsum/adler32.py"""

    name: str = "adler32"

    def int_to_hex(self, value: int) -> str:
        # return hex(value)[2:]
        return "".join([f"{x:02x}" for x in struct.pack(">I", value)]).lower()

    def hex_to_int(self, value: str) -> int:
        return int(value, 16)

    def calculate(self, file_buffer: Iterable[Any]) -> str:
        value = 1
        bytes_read = 0
        number_of_buffers_read = 0
        for buffer in file_buffer:
            value = zlib.adler32(buffer, value)
            bytes_read += len(buffer)
            number_of_buffers_read += 1
            log.trace(
                "%s: %s %s %s",
                self.name,
                self.int_to_hex(value),
                len(buffer),
                bytes_read,
            )

        self.value = self.int_to_hex(value)
        self.bytes_read = bytes_read
        self.number_of_buffers_read = number_of_buffers_read

        return self.value



"""Implementation of the HDFS backend."""
# we want non-top-level imports to avoid pulling HDFS dependency early
# pylint: disable=import-outside-toplevel
from __future__ import annotations

import xml.etree.ElementTree as ET
from collections.abc import Generator
from contextlib import closing
from dataclasses import dataclass
from typing import IO, Any

from codetiming import Timer

from ..checksums import Checksum
from ..logger import APP_LOGGER_NAME, get_logger
from ._base import XrdsumBackend

log = get_logger(APP_LOGGER_NAME)

CONF = "/etc/hadoop/conf/hdfs-site.xml"
USER = "xrootd"
XATTR_TEMPLATE = "user.xrdsum.{}"


@dataclass
class HDFSSettings:
    """Settings for the HDFS backend."""

    config_file: str = CONF
    user: str = USER
    read_size: int = 64 * 1024 * 1024


def __get_namenodes() -> list[str]:
    """
    Get the list of namenodes from the HDFS configuration file.
    """

    tree = ET.parse(CONF)
    root = tree.getroot()
    namenodes = []

    for prop in root.findall("property"):
        name = prop.find("name")
        if name is None:
            continue
        name_str = str(name.text)

        if name_str.startswith("dfs.namenode.http-address"):
            value = prop.find("value")
            if value is None:
                continue
            namenodes.append(str(value.text))
    return namenodes


def get_hdfs_client() -> Any:
    """Retrieving the HDFS client to execute operations on HDFS."""
    import pyhdfs

    namenodes = __get_namenodes()
    log.debug("Connecting to HDFS via %s", namenodes)
    return pyhdfs.HdfsClient(namenodes, user_name=USER)


def read_file_in_chunks(
    file_path: str, chunk_size_in_bytes: int
) -> Generator[IO[bytes], None, None]:
    """Reads HDFS file in chunks."""
    client = get_hdfs_client()
    file_status = client.get_file_status(file_path)
    total_size = file_status.length
    read_bytes = 0
    log.debug(
        "Reading %s in chunks of %s bytes (out of %s)",
        file_path,
        chunk_size_in_bytes,
        total_size,
    )
    with closing(client.open(file_path)) as file_handle:
        while True:
            chunk = file_handle.read(chunk_size_in_bytes)
            if chunk:
                read_bytes += len(chunk)
                yield chunk
            else:
                return


class HDFSBackend(XrdsumBackend):
    """Implementation of the HDFS backend."""

    client: Any
    settings: HDFSSettings

    def __init__(self, file_path: str, read_size: int, **kwargs: dict[str, Any]):
        """HDFS backend requires at least the file_path and read_size"""
        self.client = get_hdfs_client()
        self.file_path = file_path
        self.settings = HDFSSettings(
            read_size=read_size,
            **kwargs,  # type: ignore[arg-type]
        )

    def _get_xattr(self, xattr_name: str) -> str:
        import pyhdfs

        try:
            xattr_value = self.client.get_xattrs(
                self.file_path, xattr_name=xattr_name, encoding="text"
            )
        except pyhdfs.HdfsIOException as error:
            # this is OK, just means the xattr does not exist
            log.debug(
                "No checksum found in metadata (%s) for file %s: %s",
                xattr_name,
                self.file_path,
                error,
            )
            return ""
        if xattr_value:
            return str(xattr_value[xattr_name])

        return ""

    def get_checksum(self, checksum: Checksum) -> Checksum:
        # check if file exists
        exists = self.client.exists(self.file_path)
        if not exists:
            log.error("File %s does not exist", self.file_path)
            return checksum
        # try to get from metadata
        xattr_name = XATTR_TEMPLATE.format(checksum.name)
        with Timer(
            text=f"HDFS get_xattr took {{:.3f}}s for {self.file_path}",
            logger=log.timing,
        ):
            xattr_value = self._get_xattr(xattr_name)
        if xattr_value:
            log.debug(
                "Found checksum %s in metadata (%s=%s) for file %s",
                checksum.name,
                xattr_name,
                xattr_value,
                self.file_path,
            )
            checksum.value = xattr_value
            return checksum
        # did not find it in metadata, try calculating it
        with Timer(
            text=f"HDFS checksum calculation took {{:.3f}}s for {self.file_path}",
            logger=log.timing,
        ):
            checksum.value = checksum.calculate(
                read_file_in_chunks(self.file_path, self.settings.read_size)
            )

        return checksum

    def store_checksum(self, checksum: Checksum, force: bool = False) -> None:
        if not checksum.value:
            checksum = self.get_checksum(checksum)

        xattr_name = XATTR_TEMPLATE.format(checksum.name)
        xattr_value = self._get_xattr(xattr_name)
        xattr_flag = "CREATE"
        if xattr_value and not force:
            log.debug(
                "Checksum already exists in metadata (%s=%s) for file %s and force=False - not overwriting",
                xattr_name,
                xattr_value,
                self.file_path,
            )
            return
        if xattr_value and force:
            xattr_flag = "REPLACE"
        self.client.set_xattr(
            self.file_path,
            xattr_name,
            checksum.value,
            encoding="text",
            flag=xattr_flag,
        )



from __future__ import annotations

import pytest

from xrdsum.storage_catalog import resolve_file_path


@pytest.fixture()
def cli_param() -> str:
    return "cms|tests/storage.xml?direct"


@pytest.mark.parametrize(
    ("lfn", "pfn"),
    [
        ("/store/user/johndoe/test.txt", "/xrootd/cms/store/user/johndoe/test.txt"),
        (
            "/xrootd/cms/store/user/johndoe/test.txt",
            "/xrootd/cms/store/user/johndoe/test.txt",
        ),
        ("/xrootd/othervo/test.txt", "/xrootd/othervo/test.txt"),
    ],
)
def test_resolve_file_path(cli_param: str, lfn: str, pfn: str) -> None:
    assert resolve_file_path(lfn, cli_param) == pfn



"""Implementation of the HDFS backend."""

# we want non-top-level imports to avoid pulling HDFS dependency early
# pylint: disable=import-outside-toplevel
from __future__ import annotations

from collections.abc import Generator
from contextlib import closing
from dataclasses import dataclass
from pathlib import Path
from typing import Any

import xattr  # type: ignore[import-untyped]
from codetiming import Timer

from ..checksums import Checksum
from ..logger import APP_LOGGER_NAME, get_logger
from ._base import XrdsumBackend

log = get_logger(APP_LOGGER_NAME)

USER = "xrootd"
XATTR_TEMPLATE = "user.xrdsum.{}"


@dataclass
class POSIXSettings:
    """Settings for the POSIX backend."""

    user: str = USER
    read_size: int = 64 * 1024 * 1024


def read_file_in_chunks(
    file_path: str, chunk_size_in_bytes: int
) -> Generator[bytes, None, None]:
    """Reads POSIX file in chunks."""
    client = Path(file_path)
    file_status = client.stat()
    total_size = file_status.st_size
    read_bytes = 0
    log.debug(
        "Reading %s in chunks of %s bytes (out of %s)",
        file_path,
        chunk_size_in_bytes,
        total_size,
    )
    with closing(client.open("rb")) as file_handle:
        while True:
            file_handle.seek(read_bytes)
            chunk = file_handle.read(chunk_size_in_bytes)
            if chunk:
                read_bytes += len(chunk)
                yield chunk
            else:
                return


class POSIXBackend(XrdsumBackend):
    """Implementation of the POSIX backend."""

    settings: POSIXSettings

    def __init__(self, file_path: str, read_size: int, **kwargs: dict[str, Any]):
        """POSIX backend requires at least the file_path and read_size"""
        self.file_path = file_path
        self.settings = POSIXSettings(
            read_size=read_size,
            **kwargs,  # type: ignore[arg-type]
        )

    def _get_xattr(self, xattr_name: str) -> str:
        try:
            xattr_value = xattr.getxattr(self.file_path, attr=xattr_name)
            xattr_value = xattr_value.decode("utf-8")
        except OSError as error:
            # this is OK, just means the xattr does not exist
            log.debug(
                "No checksum found in metadata (%s) for file %s: %s",
                xattr_name,
                self.file_path,
                error,
            )
            return ""
        if xattr_value:
            return str(xattr_value)

        return ""

    def get_checksum(self, checksum: Checksum) -> Checksum:
        # check if file exists
        exists = Path(self.file_path).exists()
        if not exists:
            log.error("File %s does not exist", self.file_path)
            return checksum
        # try to get from metadata
        xattr_name = XATTR_TEMPLATE.format(checksum.name)
        with Timer(
            text=f"POSIX get_xattr took {{:.3f}}s for {self.file_path}",
            logger=log.timing,
        ):
            xattr_value = self._get_xattr(xattr_name)
        if xattr_value:
            log.debug(
                "Found checksum %s in metadata (%s=%s) for file %s",
                checksum.name,
                xattr_name,
                xattr_value,
                self.file_path,
            )
            checksum.value = xattr_value
            return checksum
        # did not find it in metadata, try calculating it
        with Timer(
            text=f"HDFS checksum calculation took {{:.3f}}s for {self.file_path}",
            logger=log.timing,
        ):
            checksum.value = checksum.calculate(
                read_file_in_chunks(self.file_path, self.settings.read_size)
            )

        return checksum

    def store_checksum(self, checksum: Checksum, force: bool = False) -> None:
        if not checksum.value:
            checksum = self.get_checksum(checksum)

        xattr_name = XATTR_TEMPLATE.format(checksum.name)
        xattr_value = self._get_xattr(xattr_name)
        if xattr_value and not force:
            log.debug(
                "Checksum already exists in metadata (%s=%s) for file %s and force=False - not overwriting",
                xattr_name,
                xattr_value,
                self.file_path,
            )
            return
        xattr.setxattr(
            self.file_path,
            xattr_name,
            checksum.value.encode("utf-8"),
        )



"""xrdsum.checksums package"""
from __future__ import annotations

from ._base import StorageCatalog
from .cms import CMSStorageCatalog

AVAILABLE_STORAGE_CATALOGS: dict[str, type[StorageCatalog]] = {
    "cms": CMSStorageCatalog,
}


def resolve_file_path(file_path: str, storage_catalog: str) -> str:
    """Resolve the file path given a file catalog."""
    if not storage_catalog:
        return file_path

    # split experiment, storage catalog and protocol
    # e.g. cms|/etc/xrootd/storage.xml?direct
    experiment, storage_catalog = storage_catalog.split("|", maxsplit=1)
    if experiment not in AVAILABLE_STORAGE_CATALOGS:
        msg = f"Unknown storage catalog {experiment}. Available: {AVAILABLE_STORAGE_CATALOGS.keys()}"
        raise ValueError(msg)

    protocol = None
    if "?" in storage_catalog:
        storage_catalog, protocol = storage_catalog.split("?", maxsplit=1)

    catalog = AVAILABLE_STORAGE_CATALOGS[experiment](storage_catalog, protocol=protocol)
    return catalog.lfn2pfn(file_path)


__all__ = [
    "CMSStorageCatalog",
    "StorageCatalog",
    "AVAILABLE_STORAGE_CATALOGS",
    "resolve_file_path",
]



"""
Backends should implement how
1. to read a file (e.g. in chunks or not)
2. to calculate the checksum of a file
3. how to store the checksum of a file (e.g. in xattr)
"""
from __future__ import annotations

from typing import Any, Protocol

from ..checksums import Checksum


class XrdsumBackend(Protocol):
    """Base protocol for backends."""

    file_path: str

    def __init__(
        self,
        file_path: str,
        read_size: int,
        **kwargs: dict[str, Any],
    ):
        raise NotImplementedError()

    def get_checksum(self, checksum: Checksum) -> Checksum:
        """Try to get checksum info from metadata, otherwise from file"""
        raise NotImplementedError()

    def store_checksum(self, checksum: Checksum, force: bool = False) -> None:
        """Store the checksum in the metadata of the file"""
        raise NotImplementedError()



"""Implementation of the CephFS backend."""
# we want non-top-level imports to avoid pulling CephFS/cephsum dependencies early
# pylint: disable=import-outside-toplevel
from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from ..checksums import Checksum
from ._base import XrdsumBackend

CONF = "/etc/ceph/ceph.conf"
KEYRING = "/etc/ceph/ceph.client.xrootd.keyring"
USER = "client.xrootd"


@dataclass
class CephSettings:
    """Settings for the CephFS backend."""

    config_file: str = CONF
    keyring: str = KEYRING
    user: str = USER
    read_size: int = 64 * 1024 * 1024


def get_ceph_client(settings: CephSettings) -> Any:
    """Retrieving the CephFS client to execute operations on CephFS."""
    import cephsum  # pylint: disable=import-error # type: ignore-imports

    return cephsum.cephtools.cluster_connect(
        conffile=settings.config_file,
        keyring=settings.keyring,
        name=settings.user,
    )


class CephFSBackend(XrdsumBackend):
    """Implementation of the CephFS backend."""

    client: Any
    settings: CephSettings

    def __init__(
        self, file_path: str, read_size: int, **kwargs: dict[str, Any]
    ) -> None:
        """CephFS backend requires at least the file_path and read_size"""
        self.file_path = file_path
        self.settings = CephSettings(
            read_size=read_size,
            **kwargs,  # type: ignore[arg-type]
        )

    def get_checksum(self, checksum: Checksum) -> Checksum:
        raise NotImplementedError()

    def store_checksum(self, checksum: Checksum, force: bool = False) -> None:
        raise NotImplementedError()



from __future__ import annotations

import pytest

from xrdsum import storage_catalog
from xrdsum.storage_catalog import AVAILABLE_STORAGE_CATALOGS

ConfigType = tuple[str, str]


@pytest.fixture()
def cms_config() -> ConfigType:
    return "tests/storage.xml", "direct"


@pytest.mark.parametrize(
    ("path", "result"),
    [
        ("/store/user/johndoe/test.txt", True),
        ("/xrootd/cms/store/user/johndoe/test.txt", False),
        ("/xrootd/othervo/test.txt", False),
    ],
)
def test_is_cms_path(cms_config: ConfigType, path: str, result: bool) -> None:
    cms = storage_catalog.CMSStorageCatalog(*cms_config)
    assert cms.is_cms_path(path) == result


@pytest.mark.parametrize(
    ("lfn", "pfn"),
    [
        ("/store/user/johndoe/test.txt", "/xrootd/cms/store/user/johndoe/test.txt"),
        (
            "/xrootd/cms/store/user/johndoe/test.txt",
            "/xrootd/cms/store/user/johndoe/test.txt",
        ),
        ("/xrootd/othervo/test.txt", "/xrootd/othervo/test.txt"),
    ],
)
def test_cms_catalog(cms_config: ConfigType, lfn: str, pfn: str) -> None:
    assert "cms" in AVAILABLE_STORAGE_CATALOGS
    storage_catalog, protocol = cms_config
    catalog = AVAILABLE_STORAGE_CATALOGS["cms"](storage_catalog, protocol)
    assert catalog.lfn2pfn(lfn) == pfn



"""Entry point for the "xrdsum" command"""
from __future__ import annotations

import logging
from typing import Any

import typer
from codetiming import Timer

from . import __version__
from .backends import FILE_SYSTEMS, XrdsumBackend
from .checksums import AVAILABLE_CHECKSUM_TYPES, Checksum
from .logger import APP_LOGGER_NAME, TRACE, get_logger
from .storage_catalog import resolve_file_path

app = typer.Typer()
log = get_logger(APP_LOGGER_NAME)


def get_file_system(file_path: str, file_system: str, read_size: int) -> XrdsumBackend:
    """Get the file system backend for the given file system"""
    try:
        fs_handle = FILE_SYSTEMS[file_system](file_path, read_size)
    except KeyError as exception:
        log.error("Unknown file system %s", file_system)
        raise typer.Exit(code=1) from exception
    return fs_handle


def get_checksum(checksum_type: str) -> Checksum:
    """Get the checksum object for the given checksum type"""
    try:
        checksum: Checksum = AVAILABLE_CHECKSUM_TYPES[checksum_type]()
    except KeyError as exception:
        log.error("Unknown checksum type %s", checksum_type)
        raise typer.Exit(code=1) from exception
    return checksum


@app.callback()
def logging_callback(
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Verbose output"),
    debug: bool = typer.Option(False, "--debug", "-d", help="Debug output"),
    log_file: str = typer.Option(None, "--log-file", "-l", help="Log file"),
) -> Any:
    """Callback to give the --verbose and --debug options to all commands"""
    # verbose is debug
    trace = verbose and debug
    verbose = debug or verbose
    debug = verbose
    log_level = logging.INFO

    if debug:
        log_level = logging.DEBUG
    if trace:
        log_level = TRACE
    get_logger(APP_LOGGER_NAME, log_level, log_file)
    log.debug("Logging to %s", log_file)


@app.command()
def get(
    file_path: str,
    store_result: bool = False,
    checksum_type: str = "adler32",
    read_size: int = typer.Option(
        default=64,
        help="""
Size [in MB] of the chunks to read from the file.
Should be a power of 2, and near (but not larger than) the stripe size.
Smaller values will use less memory, larger sizes may have benefits in IO performance.
""",
    ),
    storage_catalog: str = typer.Option(
        default="cms|/etc/xrootd/storage.xml?direct",
        help="Path to the CMS storage catalog",
    ),
    file_system: str = typer.Option(
        default="HDFS",
        help="File system to use for reading the file",
    ),
) -> None:
    """
    Get the checksum of a file.
    """
    # convert from MB to bytes
    read_size *= 1024 * 1024
    file_path = resolve_file_path(file_path, storage_catalog=storage_catalog)
    fs_handle = get_file_system(file_path, file_system, read_size)
    checksum = get_checksum(checksum_type)
    with Timer(
        text=f"HDFS checksum took {{:.3f}}s for {file_path}",
        logger=log.timing,
    ):
        checksum = fs_handle.get_checksum(checksum)
    if store_result:
        with Timer(
            text=f"Storing checksum took {{:.3f}}s for {file_path}",
            logger=log.timing,
        ):
            fs_handle.store_checksum(checksum)
    typer.echo(checksum.value)


@app.command()
def version() -> None:
    """Print the version number"""
    typer.echo(__version__)


def main() -> Any:
    """Entry point for the "xrdsum" command"""
    return app()



"""
Copyright (c) 2022 Luke Kreczko. All rights reserved.

xrdsum: Package for retrieving and calculating checksums for XRootD
"""


from __future__ import annotations

from ._version import version as __version__

__all__ = ("__version__",)



"""
Base module for storage catalog classes.
"""
from __future__ import annotations

from typing import Protocol


class StorageCatalog(Protocol):
    """
    Base definition for storage catalog classes.
    A storage catalog needs to implement the following methods:
    - __init__, which takes a config file path and protocol as an arguments
    - lfntopfn, which takes a logical filename and returns a physical filename"""

    def __init__(self, config: str | None = None, protocol: str | None = None) -> None:
        raise NotImplementedError()

    def lfn2pfn(self, lfn: str) -> str:
        """Converts logical filename to physical filename"""
        raise NotImplementedError()



"""Implementation of the HTTP backend"""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any

import requests
from codetiming import Timer

from ..checksums import Checksum
from ..logger import APP_LOGGER_NAME, get_logger
from ._base import XrdsumBackend

log = get_logger(APP_LOGGER_NAME)


@dataclass
class HttpSettings:
    """Settings for the CephFS backend."""

    base_path: str = "/storage/hdfs/checksum"
    server: str = "http://localhost:4935"
    read_size: int = 128


class HttpBackend(XrdsumBackend):
    """Queries checksums over HTTP.
    This assumes the HTTP services has reasonable defaults"""

    def __init__(
        self, file_path: str, read_size: int, **kwargs: dict[str, Any]
    ) -> None:
        """CephFS backend requires at least the file_path and read_size"""
        self.file_path = file_path
        self.settings = HttpSettings(
            read_size=read_size,
            **kwargs,  # type: ignore[arg-type]
        )

    def get_checksum(self, checksum: Checksum) -> Checksum:
        url = f"{self.settings.server}{self.settings.base_path}/{checksum.name}"
        params = {"path": self.file_path}
        with Timer(
            text=f"HTTP checksum retrieval took {{:.3f}}s for {self.file_path}",
            logger=log.timing,
        ):
            response = requests.get(url, params=params, timeout=60 * 5)
        response.raise_for_status()  # This will raise an exception for HTTP errors

        # Assuming the response is a plain text with the checksum
        checksum.value = response.text.strip()
        return checksum

    def store_checksum(self, checksum: Checksum, force: bool = False) -> None:
        """For HTTP, let's keep it simple and assume storage is done on retrieval"""
        log.trace(
            "Calling %s.store_checksum(%s, %s)",
            self.__class__.__name__,
            checksum.name,
            force,
        )
        log.debug("No store_checksum available for %s", self.__class__.__name__)



"""Application level logger for xrdsum"""
from __future__ import annotations

import logging

from fasthep_logging import TIMING, TRACE
from fasthep_logging import get_logger as flogger
from fasthep_logging._logging import FASTHEPLogger

APP_LOGGER_NAME = "xrdsum"


def get_logger(
    logger_name: str = APP_LOGGER_NAME,
    default_level: int = logging.INFO,
    log_file: str | None = None,
) -> FASTHEPLogger:
    """Wrapper for fasthep_logging.get_logger.
    Can be removed one issue
    https://github.com/FAST-HEP/fasthep-logging/issues/11
    is solved."""
    flogger(logger_name, default_level, log_file)
    return FASTHEPLogger(logger_name, default_level)


__all__ = ["APP_LOGGER_NAME", "get_logger", "TIMING", "TRACE"]
